@@ -65,9 +65,9 @@ struct _FsearchApplicationWindow {
 
     GtkWidget *statusbar;
 
-    FsearchResultView *result_view;
+    char *active_filter_name;
 
-    guint sort_overlay_timeout_id;
+    FsearchResultView *result_view;
 };
 
 typedef enum {
@@ -89,20 +89,14 @@ static void
 show_overlay(FsearchApplicationWindow *win, FsearchOverlay overlay);
 
 static void
-sort_overlay_remove_timeout(FsearchApplicationWindow *win) {
-    if (win->sort_overlay_timeout_id) {
-        g_source_remove(win->sort_overlay_timeout_id);
-        win->sort_overlay_timeout_id = 0;
-    }
-}
+on_filter_combobox_changed(GtkComboBox *widget, gpointer user_data);
 
 static FsearchFilter *
 get_active_filter(FsearchApplicationWindow *win) {
     FsearchApplication *app = FSEARCH_APPLICATION_DEFAULT;
+    FsearchConfig *config = fsearch_application_get_config(app);
     const uint32_t active_filter = gtk_combo_box_get_active(GTK_COMBO_BOX(win->filter_combobox));
-    GList *filter_element = g_list_nth(fsearch_application_get_filters(app), active_filter);
-    FsearchFilter *filter = filter_element->data;
-    return filter;
+    return fsearch_filter_manager_get_filter(config->filters, active_filter);
 }
 
 static FsearchQueryFlags
@@ -172,8 +166,8 @@ static void
 database_scan_started(FsearchApplicationWindow *win) {
     show_overlay(win, OVERLAY_DATABASE_UPDATING);
 
-    GtkWidget *cancel_update_button =
-        gtk_stack_get_child_by_name(GTK_STACK(win->popover_update_button_stack), "cancel_database_update");
+    GtkWidget *cancel_update_button = gtk_stack_get_child_by_name(GTK_STACK(win->popover_update_button_stack),
+                                                                  "cancel_database_update");
     if (cancel_update_button) {
         gtk_stack_set_visible_child(GTK_STACK(win->popover_update_button_stack), cancel_update_button);
     }
@@ -227,6 +221,34 @@ fsearch_window_set_overlay_for_database_state(FsearchApplicationWindow *win) {
     }
 }
 
+static void
+apply_filter_config(FsearchApplicationWindow *win) {
+    FsearchApplication *app = FSEARCH_APPLICATION_DEFAULT;
+    FsearchConfig *config = fsearch_application_get_config(app);
+    g_signal_handlers_block_by_func(win->filter_combobox, on_filter_combobox_changed, win);
+    gtk_combo_box_text_remove_all(GTK_COMBO_BOX_TEXT(win->filter_combobox));
+
+    uint32_t active_filter = 0;
+    for (uint32_t i = 0; i < fsearch_filter_manager_get_num_filters(config->filters); ++i) {
+        FsearchFilter *filter = fsearch_filter_manager_get_filter(config->filters, i);
+        if (filter && filter->name) {
+            if (win->active_filter_name && !strcmp(win->active_filter_name, filter->name)) {
+                // in order to restore the previously active filter we remember the index of the filter which matches
+                // the active filter name
+                active_filter = i;
+            }
+            gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(win->filter_combobox), NULL, filter->name);
+            g_clear_pointer(&filter, fsearch_filter_unref);
+        }
+    }
+    g_signal_handlers_unblock_by_func(win->filter_combobox, on_filter_combobox_changed, win);
+    gtk_combo_box_set_active(GTK_COMBO_BOX(win->filter_combobox), (int32_t)active_filter);
+
+    if (win->result_view && win->result_view->database_view) {
+        db_view_set_filters(win->result_view->database_view, config->filters);
+    }
+}
+
 static void
 fsearch_window_apply_config(FsearchApplicationWindow *self) {
     g_assert(FSEARCH_IS_APPLICATION_WINDOW(self));
@@ -238,15 +260,7 @@ fsearch_window_apply_config(FsearchApplicationWindow *self) {
     }
     fsearch_application_window_apply_search_revealer_config(self);
     fsearch_application_window_apply_statusbar_revealer_config(self);
-
-    gtk_combo_box_text_remove_all(GTK_COMBO_BOX_TEXT(self->filter_combobox));
-    for (GList *f = fsearch_application_get_filters(app); f != NULL; f = f->next) {
-        FsearchFilter *filter = f->data;
-        if (filter && filter->name) {
-            gtk_combo_box_text_append(GTK_COMBO_BOX_TEXT(self->filter_combobox), NULL, filter->name);
-        }
-    }
-    gtk_combo_box_set_active(GTK_COMBO_BOX(self->filter_combobox), 0);
+    apply_filter_config(self);
 
     fsearch_window_set_overlay_for_database_state(self);
 }
@@ -283,6 +297,7 @@ fsearch_application_window_finalize(GObject *object) {
     FsearchApplicationWindow *self = (FsearchApplicationWindow *)object;
     g_assert(FSEARCH_IS_APPLICATION_WINDOW(self));
 
+    g_clear_pointer(&self->active_filter_name, free);
     g_clear_pointer(&self->result_view->database_view, db_view_unref);
     g_clear_pointer(&self->result_view, fsearch_result_view_free);
 
@@ -326,10 +341,26 @@ static void
 fsearch_window_db_view_apply_changes(FsearchApplicationWindow *win) {
     db_view_lock(win->result_view->database_view);
     const uint32_t num_rows = is_empty_search(win) ? 0 : db_view_get_num_entries(win->result_view->database_view);
+
     win->result_view->sort_order = db_view_get_sort_order(win->result_view->database_view);
+    win->result_view->sort_type = db_view_get_sort_type(win->result_view->database_view);
+
+    FsearchQuery *query = db_view_get_query(win->result_view->database_view);
+    if (query) {
+        fsearch_statusbar_set_revealer_visibility(FSEARCH_STATUSBAR(win->statusbar),
+                                                  FSEARCH_STATUSBAR_REVEALER_SMART_MATCH_CASE,
+                                                  query->triggers_auto_match_case);
+        fsearch_statusbar_set_revealer_visibility(FSEARCH_STATUSBAR(win->statusbar),
+                                                  FSEARCH_STATUSBAR_REVEALER_SMART_SEARCH_IN_PATH,
+                                                  query->triggers_auto_match_path);
+        g_clear_pointer(&query, fsearch_query_unref);
+    }
+
     db_view_unlock(win->result_view->database_view);
 
-    win->result_view->sort_type = fsearch_list_view_get_sort_type(win->result_view->list_view);
+    fsearch_statusbar_set_num_search_results(FSEARCH_STATUSBAR(win->statusbar), num_rows);
+
+    fsearch_result_view_row_cache_reset(win->result_view);
     fsearch_list_view_set_config(win->result_view->list_view,
                                  num_rows,
                                  win->result_view->sort_order,
@@ -342,36 +373,20 @@ fsearch_window_db_view_sort_finished_cb(gpointer data) {
     FsearchApplicationWindow *win = get_window_for_id(win_id);
 
     if (win) {
-        sort_overlay_remove_timeout(win);
-        fsearch_window_set_overlay_for_database_state(win);
-
         fsearch_window_db_view_apply_changes(win);
     }
 
     return G_SOURCE_REMOVE;
 }
 
-static gboolean
-on_sort_overlay_show(gpointer data) {
-    const guint win_id = GPOINTER_TO_UINT(data);
-    FsearchApplicationWindow *win = get_window_for_id(win_id);
-    if (win) {
-        show_overlay(win, OVERLAY_RESULTS_SORTING);
-        win->sort_overlay_timeout_id = 0;
-    }
-    return G_SOURCE_REMOVE;
-}
-
 static gboolean
 fsearch_window_db_view_sort_started_cb(gpointer data) {
     const guint win_id = GPOINTER_TO_UINT(data);
     FsearchApplicationWindow *win = get_window_for_id(win_id);
-
     if (win) {
-        fsearch_window_listview_set_empty(win);
-        sort_overlay_remove_timeout(win);
-        win->sort_overlay_timeout_id = g_timeout_add(30, on_sort_overlay_show, data);
+        fsearch_statusbar_set_sort_status_delayed(FSEARCH_STATUSBAR(win->statusbar));
     }
+
     return G_SOURCE_REMOVE;
 }
 
@@ -405,24 +420,6 @@ perform_search(FsearchApplicationWindow *win) {
 
     const gchar *text = get_query_text(win);
     db_view_set_query_text(win->result_view->database_view, text);
-
-    bool reveal_smart_case = false;
-    bool reveal_smart_path = false;
-    if (!fs_str_is_empty(text)) {
-        const bool has_separator = strchr(text, G_DIR_SEPARATOR) ? 1 : 0;
-        const bool has_upper_text = fs_str_has_upper(text) ? 1 : 0;
-        FsearchApplication *app = FSEARCH_APPLICATION_DEFAULT;
-        FsearchConfig *config = fsearch_application_get_config(app);
-        reveal_smart_case = config->auto_match_case && !config->match_case && has_upper_text;
-        reveal_smart_path = config->auto_search_in_path && !config->search_in_path && has_separator;
-    }
-
-    fsearch_statusbar_set_revealer_visibility(FSEARCH_STATUSBAR(win->statusbar),
-                                              FSEARCH_STATUSBAR_REVEALER_SMART_MATCH_CASE,
-                                              reveal_smart_case);
-    fsearch_statusbar_set_revealer_visibility(FSEARCH_STATUSBAR(win->statusbar),
-                                              FSEARCH_STATUSBAR_REVEALER_SMART_SEARCH_IN_PATH,
-                                              reveal_smart_path);
 }
 
 typedef struct {
@@ -446,26 +443,13 @@ count_results_cb(gpointer key, gpointer value, count_results_ctx *ctx) {
 }
 
 static gboolean
-on_fsearch_list_view_popup(FsearchListView *view, int row_idx, gpointer user_data) {
+on_fsearch_list_view_popup(FsearchListView *view, gpointer user_data) {
     FsearchApplicationWindow *win = user_data;
     if (!win->result_view->database_view) {
         return FALSE;
     }
 
-    db_view_lock(win->result_view->database_view);
-    GString *name = db_view_entry_get_name_for_idx(win->result_view->database_view, row_idx);
-    FsearchDatabaseEntryType type = db_view_entry_get_type_for_idx(win->result_view->database_view, row_idx);
-    db_view_unlock(win->result_view->database_view);
-
-    if (!name) {
-        return FALSE;
-    }
-
-    const gboolean res = listview_popup_menu(user_data, name->str, type);
-
-    g_string_free(g_steal_pointer(&name), TRUE);
-
-    return res;
+    return listview_popup_menu(user_data, win->result_view->database_view);
 }
 
 static gboolean
@@ -485,9 +469,9 @@ on_listview_key_press_event(GtkWidget *widget, GdkEvent *event, gpointer user_da
         switch (keyval) {
         case GDK_KEY_C:
             g_action_group_activate_action(group, "copy_filepath_clipboard", NULL);
-            return TRUE;
+            return GDK_EVENT_STOP;
         default:
-            return FALSE;
+            return GDK_EVENT_PROPAGATE;
         }
     }
     else if ((state & default_modifiers) == GDK_CONTROL_MASK) {
@@ -495,54 +479,44 @@ on_listview_key_press_event(GtkWidget *widget, GdkEvent *event, gpointer user_da
         case GDK_KEY_Return:
         case GDK_KEY_KP_Enter:
             g_action_group_activate_action(group, "open_folder", NULL);
-            return TRUE;
+            return GDK_EVENT_STOP;
         case GDK_KEY_c:
             g_action_group_activate_action(group, "copy_clipboard", NULL);
-            return TRUE;
+            return GDK_EVENT_STOP;
         case GDK_KEY_x:
             g_action_group_activate_action(group, "cut_clipboard", NULL);
-            return TRUE;
+            return GDK_EVENT_STOP;
         default:
-            return FALSE;
+            return GDK_EVENT_PROPAGATE;
         }
     }
     else if ((state & default_modifiers) == GDK_SHIFT_MASK) {
         switch (keyval) {
         case GDK_KEY_Delete:
             g_action_group_activate_action(group, "delete_selection", NULL);
-            return TRUE;
+            return GDK_EVENT_STOP;
         default:
-            return FALSE;
+            return GDK_EVENT_PROPAGATE;
         }
     }
     else {
         switch (keyval) {
         case GDK_KEY_Delete:
             g_action_group_activate_action(group, "move_to_trash", NULL);
-            return TRUE;
+            return GDK_EVENT_STOP;
         case GDK_KEY_Return:
         case GDK_KEY_KP_Enter:
             g_action_group_activate_action(group, "open", NULL);
-            return TRUE;
+            return GDK_EVENT_STOP;
         default:
-            return FALSE;
+            return GDK_EVENT_PROPAGATE;
         }
     }
-    return FALSE;
+    return GDK_EVENT_PROPAGATE;
 }
 
 static void
-on_file_open_failed_response(GtkDialog *dialog, GtkResponseType response, gpointer user_data) {
-    GString *path = user_data;
-    gtk_widget_destroy(GTK_WIDGET(dialog));
-    g_string_free(path, TRUE);
-}
-
-static void
-on_fsearch_list_view_row_activated(FsearchListView *view,
-                                   FsearchDatabaseIndexType col,
-                                   int row_idx,
-                                   gpointer user_data) {
+on_fsearch_list_view_row_activated(FsearchListView *view, FsearchDatabaseIndexType col, int row_idx, gpointer user_data) {
     FsearchApplicationWindow *self = user_data;
     if (!self->result_view->database_view) {
         return;
@@ -554,45 +528,8 @@ on_fsearch_list_view_row_activated(FsearchListView *view,
         launch_folder = true;
     }
 
-    GString *path = NULL;
-    GString *path_full = NULL;
-
-    db_view_lock(self->result_view->database_view);
-    path = db_view_entry_get_path_for_idx(self->result_view->database_view, row_idx);
-    path_full = db_view_entry_get_path_full_for_idx(self->result_view->database_view, row_idx);
-    db_view_unlock(self->result_view->database_view);
-
-    if (!path) {
-        goto out;
-    }
-    if (!path_full) {
-        goto out;
-    }
-
-    if (!launch_folder ? fsearch_file_utils_launch(path_full)
-                       : fsearch_file_utils_launch_with_command(path, path_full, config->folder_open_cmd)) {
-        // open succeeded
-        fsearch_window_action_after_file_open(true);
-    }
-    else if (config->show_dialog_failed_opening) {
-        // open failed
-        ui_utils_run_gtk_dialog_async(GTK_WIDGET(self),
-                                      GTK_MESSAGE_WARNING,
-                                      GTK_BUTTONS_OK,
-                                      _("Failed to open:"),
-                                      path_full->str,
-                                      G_CALLBACK(on_file_open_failed_response),
-                                      path_full);
-        path_full = NULL;
-    }
-
-out:
-    if (path) {
-        g_string_free(g_steal_pointer(&path), TRUE);
-    }
-    if (path_full) {
-        g_string_free(g_steal_pointer(&path_full), TRUE);
-    }
+    fsearch_window_action_open_generic(self, launch_folder ? true : false, true);
+    return;
 }
 
 static void
@@ -643,7 +580,7 @@ fsearch_list_view_draw_row(cairo_t *cr,
         return;
     }
 
-    fsearch_result_view_draw_row(win->result_view->database_view,
+    fsearch_result_view_draw_row(win->result_view,
                                  cr,
                                  bin_window,
                                  layout,
@@ -658,15 +595,15 @@ fsearch_list_view_draw_row(cairo_t *cr,
 }
 
 static void
-fsearch_results_sort_func(int sort_order, gpointer user_data) {
+fsearch_results_sort_func(int sort_order, GtkSortType sort_type, gpointer user_data) {
     FsearchApplicationWindow *win = FSEARCH_APPLICATION_WINDOW(user_data);
     if (!win->result_view->database_view) {
         return;
     }
-    win->result_view->sort_type = fsearch_list_view_get_sort_type(win->result_view->list_view);
-    win->result_view->sort_order = sort_order;
+    // win->result_view->sort_type = fsearch_list_view_get_sort_type(win->result_view->list_view);
+    // win->result_view->sort_order = sort_order;
 
-    db_view_set_sort_order(win->result_view->database_view, win->result_view->sort_order);
+    db_view_set_sort_order(win->result_view->database_view, sort_order, sort_type);
 }
 
 static void
@@ -721,10 +658,10 @@ add_columns(FsearchListView *view, FsearchConfig *config) {
     fsearch_list_view_append_column(FSEARCH_LIST_VIEW(view), type_col);
     fsearch_list_view_append_column(FSEARCH_LIST_VIEW(view), size_col);
     fsearch_list_view_append_column(FSEARCH_LIST_VIEW(view), changed_col);
-    fsearch_list_view_column_set_tooltip(
-        type_col,
-        _("Sorting by <b>Type</b> can be very slow with many results and it can't be aborted.\n\n"
-          "This sort order is not persistent, it will be reset when the search term changes."));
+    fsearch_list_view_column_set_tooltip(type_col,
+                                         _("Sorting by <b>Type</b> can take a few seconds with many results.\n\n"
+                                           "This sort order is not persistent, it will be reset when the search term "
+                                           "changes."));
     fsearch_list_view_column_set_emblem(type_col, "emblem-important-symbolic", TRUE);
 }
 
@@ -742,6 +679,14 @@ on_listview_row_unselect_all(gpointer user_data) {
     }
 }
 
+static void
+on_listview_row_toggle_range(int start_row, int end_row, gpointer user_data) {
+    FsearchApplicationWindow *win = FSEARCH_APPLICATION_WINDOW(user_data);
+    if (win->result_view->database_view) {
+        db_view_toggle_range(win->result_view->database_view, start_row, end_row);
+    }
+}
+
 static void
 on_listview_row_select_range(int start_row, int end_row, gpointer user_data) {
     FsearchApplicationWindow *win = FSEARCH_APPLICATION_WINDOW(user_data);
@@ -781,7 +726,8 @@ static void
 fsearch_application_window_init_overlays(FsearchApplicationWindow *win) {
     g_assert(FSEARCH_IS_APPLICATION_WINDOW(win));
 
-    GtkBuilder *builder = gtk_builder_new_from_resource("/io/github/cboxdoerfer/fsearch/ui/fsearch_overlay.ui");
+    g_autoptr(GtkBuilder) builder = gtk_builder_new_from_resource("/io/github/cboxdoerfer/fsearch/ui/"
+                                                                  "fsearch_overlay.ui");
 
     win->main_database_overlay_stack = GTK_WIDGET(gtk_builder_get_object(builder, "main_database_overlay_stack"));
     win->main_search_overlay_stack = GTK_WIDGET(gtk_builder_get_object(builder, "main_search_overlay_stack"));
@@ -811,8 +757,6 @@ fsearch_application_window_init_overlays(FsearchApplicationWindow *win) {
     gtk_stack_set_visible_child(GTK_STACK(win->main_stack), win->main_database_overlay_stack);
 
     gtk_widget_show_all(win->main_stack);
-
-    g_clear_object(&builder);
 }
 
 static void
@@ -835,24 +779,17 @@ fsearch_application_window_init_listview(FsearchApplicationWindow *win) {
                                              on_listview_row_select,
                                              on_listview_row_select_toggle,
                                              on_listview_row_select_range,
+                                             on_listview_row_toggle_range,
                                              on_listview_row_unselect_all,
                                              on_listview_row_num_selected,
                                              win);
     fsearch_list_view_set_single_click_activate(list_view, config->single_click_open);
-    fsearch_list_view_set_sort_type(list_view,
-                                    config->restore_column_config
-                                        ? (config->sort_ascending ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING)
-                                        : GTK_SORT_ASCENDING);
     gtk_widget_set_has_tooltip(GTK_WIDGET(list_view), config->enable_list_tooltips);
 
     add_columns(list_view, config);
 
     g_signal_connect_object(list_view, "row-popup", G_CALLBACK(on_fsearch_list_view_popup), win, G_CONNECT_AFTER);
-    g_signal_connect_object(list_view,
-                            "row-activated",
-                            G_CALLBACK(on_fsearch_list_view_row_activated),
-                            win,
-                            G_CONNECT_AFTER);
+    g_signal_connect_object(list_view, "row-activated", G_CALLBACK(on_fsearch_list_view_row_activated), win, G_CONNECT_AFTER);
     g_signal_connect(list_view, "key-press-event", G_CALLBACK(on_listview_key_press_event), win);
 
     win->result_view->list_view = list_view;
@@ -863,12 +800,10 @@ on_database_update_finished(gpointer data, gpointer user_data) {
     FsearchApplicationWindow *win = (FsearchApplicationWindow *)user_data;
     g_assert(FSEARCH_IS_APPLICATION_WINDOW(win));
 
-    fsearch_statusbar_set_query_text(FSEARCH_STATUSBAR(win->statusbar), "");
+    fsearch_statusbar_set_num_search_results(FSEARCH_STATUSBAR(win->statusbar), 0);
 
-    fsearch_application_window_unselect_all(win);
-
-    GtkWidget *update_database_button =
-        gtk_stack_get_child_by_name(GTK_STACK(win->popover_update_button_stack), "update_database");
+    GtkWidget *update_database_button = gtk_stack_get_child_by_name(GTK_STACK(win->popover_update_button_stack),
+                                                                    "update_database");
     if (update_database_button) {
         gtk_stack_set_visible_child(GTK_STACK(win->popover_update_button_stack), update_database_button);
     }
@@ -876,8 +811,7 @@ on_database_update_finished(gpointer data, gpointer user_data) {
 
     FsearchDatabase *db = fsearch_application_get_db(FSEARCH_APPLICATION_DEFAULT);
 
-    db_view_unregister(win->result_view->database_view);
-    db_view_register(db, win->result_view->database_view);
+    db_view_register_database(win->result_view->database_view, db);
 
     g_clear_pointer(&db, db_unref);
 }
@@ -915,11 +849,7 @@ fsearch_application_window_init(FsearchApplicationWindow *self) {
 
     FsearchApplication *app = FSEARCH_APPLICATION_DEFAULT;
     g_signal_connect_object(app, "database-scan-started", G_CALLBACK(on_database_scan_started), self, G_CONNECT_AFTER);
-    g_signal_connect_object(app,
-                            "database-update-finished",
-                            G_CALLBACK(on_database_update_finished),
-                            self,
-                            G_CONNECT_AFTER);
+    g_signal_connect_object(app, "database-update-finished", G_CALLBACK(on_database_update_finished), self, G_CONNECT_AFTER);
     g_signal_connect_object(app, "database-load-started", G_CALLBACK(on_database_load_started), self, G_CONNECT_AFTER);
 }
 
@@ -930,10 +860,15 @@ on_filter_combobox_changed(GtkComboBox *widget, gpointer user_data) {
 
     int active = gtk_combo_box_get_active(GTK_COMBO_BOX(win->filter_combobox));
     char *active_filter_name = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(win->filter_combobox));
+    if (active_filter_name) {
+        g_clear_pointer(&win->active_filter_name, free);
+        win->active_filter_name = active_filter_name;
+    }
     fsearch_statusbar_set_filter(FSEARCH_STATUSBAR(win->statusbar), active ? active_filter_name : NULL);
-    g_clear_pointer(&active_filter_name, g_free);
 
-    db_view_set_filter(win->result_view->database_view, get_active_filter(win));
+    FsearchFilter *filter = get_active_filter(win);
+    db_view_set_filter(win->result_view->database_view, filter);
+    g_clear_pointer(&filter, fsearch_filter_unref);
 }
 
 static gboolean
@@ -1017,10 +952,6 @@ fsearch_window_db_view_content_changed_cb(gpointer data) {
     const uint32_t num_rows = is_empty_search(win) ? 0 : db_view_get_num_entries(win->result_view->database_view);
     db_view_unlock(win->result_view->database_view);
 
-    gchar sb_text[100] = "";
-    snprintf(sb_text, sizeof(sb_text), _("%'d Items"), num_rows);
-    fsearch_statusbar_set_query_text(FSEARCH_STATUSBAR(win->statusbar), sb_text);
-
     if (is_empty_search(win)) {
         show_overlay(win, OVERLAY_QUERY_EMPTY);
         gtk_widget_show(win->main_search_overlay_stack);
@@ -1144,11 +1075,16 @@ fsearch_application_window_apply_search_revealer_config(FsearchApplicationWindow
 
 void
 fsearch_application_window_update_query_flags(FsearchApplicationWindow *win) {
+    apply_filter_config(win);
     db_view_set_query_flags(win->result_view->database_view, get_query_flags());
 }
 
 static FsearchDatabaseIndexType
 get_sort_type_for_name(const char *name) {
+    if (!name) {
+        g_warning("[get_sort_type_for_name] name is nullptr");
+        return DATABASE_INDEX_TYPE_NAME;
+    }
     if (!strcmp(name, DATABASE_INDEX_TYPE_NAME_STRING)) {
         return DATABASE_INDEX_TYPE_NAME;
     }
@@ -1207,20 +1143,54 @@ fsearch_application_window_prepare_shutdown(gpointer self) {
     FsearchApplication *app = FSEARCH_APPLICATION_DEFAULT;
     FsearchConfig *config = fsearch_application_get_config(app);
 
-    gint width = 800;
+    gint width = 850;
     gint height = 800;
     gtk_window_get_size(GTK_WINDOW(self), &width, &height);
     config->window_width = width;
     config->window_height = height;
 
-    if (win->result_view && win->result_view->list_view) {
-        config->sort_ascending =
-            fsearch_list_view_get_sort_type(win->result_view->list_view) == GTK_SORT_ASCENDING ? true : false;
+    if (win->result_view) {
+        if (win->result_view->database_view) {
+            FsearchDatabaseView *db_view = win->result_view->database_view;
+            db_view_lock(db_view);
+            config->sort_ascending = db_view_get_sort_type(db_view) == GTK_SORT_ASCENDING ? true : false;
 
-        if (config->sort_by) {
-            g_clear_pointer(&config->sort_by, g_free);
+            if (config->sort_by) {
+                g_clear_pointer(&config->sort_by, g_free);
+            }
+            config->sort_by = get_sort_name_for_type(db_view_get_sort_order(db_view));
+            db_view_unlock(db_view);
+        }
+
+        if (win->result_view->list_view) {
+            FsearchListView *list_view = win->result_view->list_view;
+            config->sort_ascending = fsearch_list_view_get_sort_type(list_view) == GTK_SORT_ASCENDING ? true : false;
+
+            if (config->sort_by) {
+                g_clear_pointer(&config->sort_by, g_free);
+            }
+            config->sort_by = get_sort_name_for_type(fsearch_list_view_get_sort_order(list_view));
+
+            // update the config with the widths of all columns whose width we can store
+            const struct {
+                int type;
+                uint32_t *width;
+            } columns[] = {
+                {DATABASE_INDEX_TYPE_NAME, &config->name_column_width},
+                {DATABASE_INDEX_TYPE_PATH, &config->path_column_width},
+                {DATABASE_INDEX_TYPE_FILETYPE, &config->type_column_width},
+                {DATABASE_INDEX_TYPE_EXTENSION, &config->extension_column_width},
+                {DATABASE_INDEX_TYPE_SIZE, &config->size_column_width},
+                {DATABASE_INDEX_TYPE_MODIFICATION_TIME, &config->modified_column_width},
+            };
+            for (int i = 0; i < G_N_ELEMENTS(columns); i++) {
+                const FsearchListViewColumn *col = fsearch_list_view_get_first_column_for_type(list_view,
+                                                                                               columns[i].type);
+                if (col) {
+                    *columns[i].width = col->width;
+                }
+            }
         }
-        config->sort_by = get_sort_name_for_type(fsearch_list_view_get_sort_order(win->result_view->list_view));
     }
 }
 
@@ -1242,28 +1212,43 @@ fsearch_application_window_added(FsearchApplicationWindow *win, FsearchApplicati
 
     FsearchConfig *config = fsearch_application_get_config(app);
 
-    FsearchDatabaseIndexType sort_order =
-        config->restore_column_config ? get_sort_type_for_name(config->sort_by) : DATABASE_INDEX_TYPE_NAME;
+    FsearchDatabaseIndexType sort_order = config->restore_sort_order ? get_sort_type_for_name(config->sort_by)
+                                                                     : DATABASE_INDEX_TYPE_NAME;
     if (sort_order == DATABASE_INDEX_TYPE_FILETYPE) {
         // file type order is not indexed, so it would make startup really slow
         // -> fall back to sort by name instead
         sort_order = DATABASE_INDEX_TYPE_NAME;
     }
+    GtkSortType sort_type = config->restore_sort_order
+                              ? (config->sort_ascending ? GTK_SORT_ASCENDING : GTK_SORT_DESCENDING)
+                              : GTK_SORT_ASCENDING;
 
+    FsearchFilter *filter = get_active_filter(win);
     win->result_view->database_view = db_view_new(get_query_text(win),
                                                   get_query_flags(),
-                                                  get_active_filter(win),
+                                                  filter,
+                                                  config->filters,
                                                   sort_order,
+                                                  sort_type,
                                                   fsearch_window_db_view_notify,
                                                   GUINT_TO_POINTER(win_id));
+    g_clear_pointer(&filter, fsearch_filter_unref);
 
     FsearchDatabase *db = fsearch_application_get_db(FSEARCH_APPLICATION_DEFAULT);
     if (db) {
-        db_view_register(db, win->result_view->database_view);
+        db_view_register_database(win->result_view->database_view, db);
         g_clear_pointer(&db, db_unref);
     }
 }
 
+void
+fsearch_application_window_cancel_current_task(FsearchApplicationWindow *win) {
+    g_assert(FSEARCH_IS_APPLICATION_WINDOW(win));
+    if (win->result_view && win->result_view->database_view) {
+        db_view_cancel_current_task(win->result_view->database_view);
+    }
+}
+
 void
 fsearch_application_window_removed(FsearchApplicationWindow *win, FsearchApplication *app) {
     g_assert(FSEARCH_IS_APPLICATION_WINDOW(win));
@@ -1310,6 +1295,13 @@ fsearch_application_window_selection_for_each(FsearchApplicationWindow *self, GH
     }
 }
 
+void
+fsearch_application_window_focus_search_entry(FsearchApplicationWindow *win) {
+    g_assert(FSEARCH_IS_APPLICATION_WINDOW(win));
+    // Make sure the entry also has focus and the text is selected
+    gtk_widget_grab_focus(win->search_entry);
+}
+
 GtkEntry *
 fsearch_application_window_get_search_entry(FsearchApplicationWindow *self) {
     g_assert(FSEARCH_IS_APPLICATION_WINDOW(self));
@@ -1323,7 +1315,7 @@ fsearch_application_window_get_statusbar(FsearchApplicationWindow *self) {
 }
 
 void
-fsearch_application_window_set_database_index_text(FsearchApplicationWindow *self, const char *text) {
+fsearch_application_window_set_database_index_progress(FsearchApplicationWindow *self, const char *text) {
     g_assert(FSEARCH_IS_APPLICATION_WINDOW(self));
     fsearch_statusbar_set_database_index_text(FSEARCH_STATUSBAR(self->statusbar), text);
 }
@@ -1356,6 +1348,8 @@ fsearch_application_window_update_listview_config(FsearchApplicationWindow *win)
     FsearchConfig *config = fsearch_application_get_config(FSEARCH_APPLICATION_DEFAULT);
     fsearch_list_view_set_single_click_activate(win->result_view->list_view, config->single_click_open);
     gtk_widget_set_has_tooltip(GTK_WIDGET(win->result_view->list_view), config->enable_list_tooltips);
+
+    fsearch_application_window_redraw_listview(win);
 }
 
 FsearchListView *
